@@ -1,18 +1,21 @@
 # -*- coding: utf-8 -*-
-try:
-    from zope.app.event.objectevent import objectEventNotify # BBB
-except ImportError, e:
-    from zope.component.event import objectEventNotify
 from odict import odict
+from ldap import (
+    MOD_ADD,
+    MOD_DELETE,
+    MOD_REPLACE,
+)
+from ldap.functions import explode_dn
 from zope.interface import implementer
+from zope.deprecation import deprecated
 from plumber import (
     plumber,
-    Part,
+    Behavior,
     plumb,
     default,
     finalize,
 )
-from node.parts import (
+from node.behaviors import (
     Nodespaces,
     Attributes,
     NodeAttributes,
@@ -20,7 +23,6 @@ from node.parts import (
     AttributesLifecycle,
     NodeChildValidate,
     Adopt,
-    UnicodeAware,
     Nodify,
     OdictStorage,
 )
@@ -31,45 +33,39 @@ from node.utils import (
     debug,
 )
 from node.interfaces import IInvalidate
-from node.ext.ldap import (
+from . import (
     BASE,
     ONELEVEL,
     LDAPSession,
 )
-from node.ext.ldap.interfaces import ILDAPStorage
-from node.ext.ldap.events import (
+from .interfaces import ILDAPStorage
+from .events import (
     LDAPNodeCreatedEvent,
     LDAPNodeAddedEvent,
     LDAPNodeModifiedEvent,
     LDAPNodeRemovedEvent,
     LDAPNodeDetachedEvent
 )
-from node.ext.ldap.filter import (
+from .filter import (
     LDAPFilter,
     LDAPDictFilter,
     LDAPRelationFilter,
 )
-from node.ext.ldap.schema import LDAPSchemaInfo
-from ldap.functions import explode_dn
-from ldap import (
-    MOD_ADD,
-    MOD_DELETE,
-    MOD_REPLACE,
-)
+from .schema import LDAPSchemaInfo
+
 
 ACTION_ADD = 0
 ACTION_MODIFY = 1
 ACTION_DELETE = 2
 
 
-class AttributesPart(Part):
+class AttributesBehavior(Behavior):
 
     @plumb
     def __init__(_next, self, name=None, parent=None):
         _next(self, name=name, parent=parent)
         self.load()
 
-
     @default
     def load(self):
         if not self.parent.name \
@@ -92,13 +88,13 @@ class AttributesPart(Part):
         # fetch our node with all attributes
         entry = self.parent.ldap_session.search(
                 scope=BASE,
-                baseDN=self.parent.DN,
+                baseDN=self.parent.DN.encode('utf-8'),
                 force_reload=self.parent._reload,
                 attrlist=attrlist,
                 )
 
         if len(entry) != 1:
-            raise RuntimeError(#pragma NO COVERAGE
+            raise RuntimeError(                             #pragma NO COVERAGE
                 u"Fatal. Expected entry does not exist or " #pragma NO COVERAGE
                 u"more than one entry found")               #pragma NO COVERAGE
 
@@ -144,10 +140,17 @@ class AttributesPart(Part):
         return name in self.parent.root._multivalued_attributes
 
 
+AttributesPart = AttributesBehavior  # B/C
+deprecated('AttributesPart', """
+``node.ext.ldap._node.AttributesPart`` is deprecated as of node.ext.ldap 0.9.4
+and will be removed in node.ext.ldap 1.0. Use
+``node.ext.ldap._node.AttributesBehavior`` instead.""")
+
+
 class LDAPNodeAttributes(NodeAttributes):
     __metaclass__ = plumber
     __plumbing__ = (
-        AttributesPart,
+        AttributesBehavior,
         AttributesLifecycle,
     )
 
@@ -350,7 +353,7 @@ class LDAPStorage(OdictStorage):
         # Doctest fails if we output utf-8
         try:
             dn = self.DN.encode('ascii', 'replace') or '(dn not set)'
-        except KeyError, e:
+        except KeyError:
             dn = '(dn not available yet)'
         if self.parent is None:
             return "<%s - %s>" % (dn, self.changed)
@@ -451,11 +454,11 @@ class LDAPStorage(OdictStorage):
                page_size=None, cookie=None):
         attrset = set(attrlist or [])
         attrset.discard('dn')
-        
+
         # fetch also the key attribute
         if not self._key_attr == 'rdn':
             attrset.add(self._key_attr)
-        
+
         # Create queryFilter from all filter definitions
         # filter for this search ANDed with the default filters defined on self
         search_filter = LDAPFilter(queryFilter)
@@ -530,14 +533,14 @@ class LDAPStorage(OdictStorage):
     @default
     def invalidate(self, key=None):
         """Invalidate LDAP node.
-        
+
         If key is None:
             - check if self is changed
             - if changed, raise RuntimeError
             - reload self.attrs
             - set self._reload to True. This reloads the keys forcing cache
               reload as well.
-        
+
         If key is given:
             - if child in self._keys, check if child has changed
             - if changed, raise RuntimeError
@@ -703,7 +706,7 @@ class LDAPStorage(OdictStorage):
                 moddef = (MOD_REPLACE, encode(key), value)
                 modlist.append(moddef)
         if modlist:
-            self.ldap_session.modify(self.DN, modlist)
+            self.ldap_session.modify(encode(self.DN), modlist)
 
     @default
     def _ldap_delete(self):
@@ -712,7 +715,7 @@ class LDAPStorage(OdictStorage):
         self.parent._keys[self.name] = False
         del self.parent.storage[self.name]
         del self.parent._keys[self.name]
-        self.ldap_session.delete(self.DN)
+        self.ldap_session.delete(encode(self.DN))
 
     @default
     @property
@@ -721,6 +724,7 @@ class LDAPStorage(OdictStorage):
             return self.root._ldap_schema_info
         return self._ldap_schema_info
 
+
 class LDAPNode(object):
     __metaclass__ = plumber
     __plumbing__ = (
@@ -732,7 +736,6 @@ class LDAPNode(object):
         Nodify,
         LDAPStorage,
     )
-
     events = {
         'created':  LDAPNodeCreatedEvent,
         'added':    LDAPNodeAddedEvent,
